@@ -42,126 +42,6 @@
 #' @family Top level API functions
 #' @name mims_unit
 
-#' @rdname mims_unit
-#' @param files character vector. A list of csv filepaths for raw accelerometer
-#'   data organized in order to be processed. The data should be consecutive in
-#'   timestamps. A typical case is a set of hourly or daily files for
-#'   continuous accelerometer sampling.
-#' @param file_type character. "mhealth" or "actigraph". The type of the csv files
-#' that store the raw accelerometer data.
-#' @param ... additional parameters passed to the import function when reading
-#'   in the data from the files.
-#' @export
-#' @examples
-#'   # Use sample mhealth file for testing
-#'   filepaths = c(
-#'     system.file('extdata', 'mhealth.csv', package='MIMSunit'),
-#'     system.file('extdata', 'mhealth1.csv', package='MIMSunit')
-#'   )
-#'
-#'   # Test with single file
-#'   mims_unit_from_files(c(filepaths[1]),
-#'                                 epoch = "10 sec",
-#'                                 dynamic_range = c(-2, 2))
-#'
-#'   # Test with multiple files
-#'   mims_unit_from_files(filepaths, epoch = "20 sec", dynamic_range = c(-2, 2))
-mims_unit_from_files <-
-  function(files,
-           epoch = "5 sec",
-           dynamic_range,
-           output_mims_per_axis = FALSE,
-           use_gui_progress = FALSE,
-           file_type = "mhealth", ...) {
-    num_of_files <- length(files)
-    dots = list(...)
-    df <- NULL
-    results <- list()
-
-    if (file_type == "mhealth") {
-      import_fun <- import_mhealth_csv_chunked
-      header = TRUE
-    } else if (file_type == "actigraph") {
-      import_fun <- function(x, chunk_samples) import_actigraph_csv_chunked(x, chunk_samples = chunk_samples, ...)
-      header = dots$header
-    } else {
-      stop('Only "mhealth" or "actigraph" file types are supported')
-    }
-
-    meta = .get_meta(files[1], file_type = file_type, header = header)
-    sr = meta$sr
-    st = meta$st
-    num_samples_epoch = parse_epoch_string(epoch, sr)
-    num_per_load = max(num_samples_epoch, 180000)
-    last_epoch_st = NULL
-    last_chunk = NULL
-    j = 1
-    for (i in 1:num_of_files) {
-      funcs = import_fun(files[i], chunk_samples = num_per_load)
-      next_chunk = funcs[[1]]
-      close_con = funcs[[2]]
-
-      repeat {
-        chunk = next_chunk()
-        if (nrow(chunk) > 0) {
-          num_per_df = nrow(chunk)
-          if (!is.null(df)) {
-            if (!is.null(last_epoch_st) & !is.null(last_chunk)) {
-              df = clip_data(last_chunk, start_time = last_epoch_st,
-                             stop_time = last_chunk[nrow(last_chunk),1])
-              df = rbind(df, chunk[1:num_per_df,])
-            } else {
-              df = chunk[1:num_per_df,]
-            }
-          } else {
-            df = chunk[1:num_per_df,]
-          }
-          result = mims_unit(df,
-                    before_df = NULL,
-                    after_df = NULL,
-                    epoch = epoch,
-                    dynamic_range = dynamic_range,
-                    output_mims_per_axis = output_mims_per_axis,
-                    use_gui_progress = use_gui_progress,
-                    st = st
-          )
-          last_epoch_st = result[nrow(result),1]
-          last_chunk = chunk
-          # discard the last epoch
-          result = result[1:(nrow(result) - 1),]
-          results[[j]] = result
-          j = j + 1
-        }
-        else {
-          close_con()
-          break
-        }
-      }
-    }
-    result <- do.call(rbind, results)
-    return(result)
-  }
-
-
-.get_meta <- function(filepath, file_type, header) {
-  if (file_type == 'actigraph') {
-    meta = import_actigraph_meta(filepath, header = header)
-    st = meta$st
-    sr = meta$sr
-    return(list(st = st, sr = sr))
-  } else if (file_type == 'mhealth') {
-    # use the first 200 rows to compute sampling rate
-    funcs = import_mhealth_csv_chunked(filepath, chunk_samples = 200)
-    next_chunk = funcs[[1]]
-    close_con = funcs[[2]]
-    df = next_chunk()
-    close_con()
-    sr = sampling_rate(df)
-    st = df[1, 1]
-    return(list(st = st, sr = sr))
-  }
-}
-
 
 #' @rdname mims_unit
 #' @param df dataframe. Input multi-channel accelerometer signal.  The
@@ -186,15 +66,8 @@ mims_unit_from_files <-
 #'   # Use sample data for testing
 #'   df = sample_raw_accel_data
 #'
-#'   # compute mims unit values
-#'   mims_unit(df, epoch = '5 min', dynamic_range=c(-8, 8))
-#'
-#'   # compute mims unit values with different epoch length
-#'   output = mims_unit(df, epoch = '15 sec', dynamic_range=c(-8, 8))
-#'   head(output)
-#'
-#'   # output axial values
-#'   output = mims_unit(df, epoch = '15 sec', dynamic_range=c(-8, 8), output_mims_per_axis=TRUE)
+#'   # compute mims unit values and output axial values
+#'   output = mims_unit(df, epoch = '1 sec', dynamic_range=c(-8, 8), output_mims_per_axis=TRUE)
 #'   head(output)
 mims_unit <-
   function(df,
@@ -281,10 +154,10 @@ mims_unit <-
 #'   df = sample_raw_accel_data
 #'
 #'   # compute sensor orientation angles
-#'   sensor_orientations(df, epoch = '5 min', dynamic_range=c(-8, 8))
+#'   sensor_orientations(df, epoch = '2 sec', dynamic_range=c(-8, 8))
 #'
 #'   # compute sensor orientation angles with different epoch length
-#'   output = sensor_orientations(df, epoch = '15 sec', dynamic_range=c(-8, 8))
+#'   output = sensor_orientations(df, epoch = '1 sec', dynamic_range=c(-8, 8))
 #'   head(output)
 sensor_orientations <-
   function(df,
@@ -415,12 +288,8 @@ sensor_orientations <-
 #'   # Use sample data for testing
 #'   df = sample_raw_accel_data
 #'
-#'   # compute mims unit values
-#'   output = custom_mims_unit(df, epoch = '15 sec', dynamic_range=c(-8, 8))
-#'   head(output)
-#'
 #'   # compute mims unit values with custom parameter
-#'   output = custom_mims_unit(df, epoch = '15 sec', dynamic_range=c(-8, 8), spar=0.7)
+#'   output = custom_mims_unit(df, epoch = '1 sec', dynamic_range=c(-8, 8), spar=0.7)
 #'   head(output)
 custom_mims_unit <-
   function(df,
@@ -454,19 +323,29 @@ custom_mims_unit <-
     rm(first_col)
 
     # save the start and stop time of original df
-    if (use_gui_progress & .Platform$OS.type == 'windows') {
+    if (shiny::isRunning()) {
+      pb = shiny::Progress$new()
+      pb$set(message = "Computing MIMS-unit values", value=0)
+
+    }
+    else if (use_gui_progress & .Platform$OS.type == 'windows') {
       ProgressBar = utils::winProgressBar
       setProgressBar = utils::setWinProgressBar
+      pb = ProgressBar(min = 0,
+                       max = 1,
+                       title = "Computing MIMS-unit values",
+                       label = 'Starting...')
     }
     else {
       ProgressBar = utils::txtProgressBar
       setProgressBar = utils::setTxtProgressBar
+      pb = ProgressBar(min = 0,
+                       max = 1,
+                       title = "Computing MIMS-unit values",
+                       label = 'Starting...')
     }
 
-    pb = ProgressBar(min = 0,
-                     max = 1,
-                     title = "Computing MIMS-unit values",
-                     label = 'Starting...')
+
 
     start_time <- lubridate::floor_date(df[[1]][1], unit = "seconds")
     stop_time <-
@@ -474,22 +353,38 @@ custom_mims_unit <-
 
     # concatenate with before and after df
     if (is.data.frame(before_df)) {
-      setProgressBar(pb, 0.2, label = 'Concatenating before_df...')
+      if (shiny::isRunning()) {
+        pb$inc(0.2, detail='Concatenating before_df...')
+      } else {
+        setProgressBar(pb, 0.2, label = 'Concatenating before_df...')
+      }
       df <- rbind(before_df, df)
     }
     if (is.data.frame(after_df)) {
-      setProgressBar(pb, 0.3, label = 'Concatenating after_df...')
+      if (shiny::isRunning()) {
+        pb$inc(0.3, detail='Concatenating after_df...')
+      } else {
+        setProgressBar(pb, 0.3, label = 'Concatenating after_df...')
+      }
       df <- rbind(df, after_df)
     }
 
     # apply extrapolation algorithm
     if (use_extrapolation) {
       stopifnot(length(dynamic_range) == 2)
-      setProgressBar(pb, 0.4, label = 'Extrapolating signal...')
+      if (shiny::isRunning()) {
+        pb$inc(0.4, detail='Extrapolating signal...')
+      } else {
+        setProgressBar(pb, 0.4, label = 'Extrapolating signal...')
+      }
       resampled_data <-
         extrapolate(df, dynamic_range, noise_level, k, spar)
     } else {
-      setProgressBar(pb, 0.4, label = 'Interpolating signal...')
+      if (shiny::isRunning()) {
+        pb$inc(0.4, detail='Interpolating signal...')
+      } else {
+        setProgressBar(pb, 0.4, label = 'Interpolating signal...')
+      }
       resampled_data <-
         interpolate_signal(df, sr = 100, method = "linear")
     }
@@ -508,9 +403,28 @@ custom_mims_unit <-
     abnormal_data <- resampled_data[row_abnormal, ]
     normal_data <- resampled_data[!row_abnormal, ]
 
+    # Compute orientations
+    if (output_orientation_estimation) {
+      if (is.null(epoch_for_orientation_estimation)) {
+        epoch_for_orientation_estimation <- epoch
+      }
+      orientation_data <-
+        aggregate_for_orientation(resampled_data,
+                                  epoch = epoch_for_orientation_estimation,
+                                  st = st
+        )
+    } else {
+      orientation_data <- NULL
+    }
+    rm(resampled_data); gc();
+
     # Apply filter cascade
     if (use_filtering) {
-      setProgressBar(pb, 0.6, label = 'Filtering signal...')
+      if (shiny::isRunning()) {
+        pb$inc(0.6, detail='Filtering signal...')
+      } else {
+        setProgressBar(pb, 0.6, label = 'Filtering signal...')
+      }
       if (filter_type == "butter") {
         filtered_data <-
           iir(
@@ -550,26 +464,17 @@ custom_mims_unit <-
     # sort by timestamp
     colnames(abnormal_data)[2:4] <- colnames(filtered_data)[2:4]
     filtered_data <- rbind(filtered_data, abnormal_data)
+    rm(abnormal_data)
     filtered_data <-
       filtered_data[order(filtered_data[[1]]), ]
 
-    # Compute orientations
-    if (output_orientation_estimation) {
-      if (is.null(epoch_for_orientation_estimation)) {
-        epoch_for_orientation_estimation <- epoch
-      }
-      orientation_data <-
-        aggregate_for_orientation(resampled_data,
-          epoch = epoch_for_orientation_estimation,
-          st = st
-        )
-      rm(resampled_data); gc();
-    } else {
-      orientation_data <- NULL
-    }
 
     # Compute the AUC
-    setProgressBar(pb, 0.8, label = 'Computing AUC...')
+    if (shiny::isRunning()) {
+      pb$inc(0.8, detail='Computing AUC...')
+    } else {
+      setProgressBar(pb, 0.8, label = 'Computing AUC...')
+    }
     integrated_data <-
       aggregate_for_mims(
         filtered_data,
@@ -581,7 +486,11 @@ custom_mims_unit <-
     rm(filtered_data);
 
     if (allow_truncation) {
-      setProgressBar(pb, 0.9, label = 'Truncating signal...')
+      if (shiny::isRunning()) {
+        pb$inc(0.9, detail='Truncating signal...')
+      } else {
+        setProgressBar(pb, 0.9, label = 'Truncating signal...')
+      }
       truncate_indices <-
         integrated_data[, 2:ncol(integrated_data)] > 0 &
           (integrated_data[, 2:ncol(integrated_data)] <=
@@ -595,7 +504,13 @@ custom_mims_unit <-
     }
 
     # Compute vector magnitude
-    setProgressBar(pb, 1, label = 'Summing up axial values...')
+    if (shiny::isRunning()) {
+      pb$inc(1, detail='Summing up axial values...')
+      pb$close()
+    } else {
+      setProgressBar(pb, 1, label = 'Summing up axial values...')
+      close(pb)
+    }
     row_abnormal <- rep(FALSE, nrow(integrated_data))
     for (i in 2:ncol(integrated_data))
     {
@@ -629,10 +544,147 @@ custom_mims_unit <-
       mims_data[[1]] >= start_time & mims_data[[1]] < stop_time
     mims_data <- mims_data[keep_mask, ]
 
-    close(pb)
+
     if (output_orientation_estimation) {
       return(list(mims = mims_data, orientation = orientation_data))
     } else {
       return(mims_data)
     }
   }
+
+#' @rdname mims_unit
+#'
+#' @param files character vector. A list of csv filepaths for raw accelerometer
+#'   data organized in order to be processed. The data should be consecutive in
+#'   timestamps. A typical case is a set of hourly or daily files for
+#'   continuous accelerometer sampling. For a single file, please wrap the filepath
+#'   in a vector `c(filepath)`.
+#' @param file_type character. "mhealth" or "actigraph". The type of the csv files
+#' that store the raw accelerometer data.
+#' @param ... additional parameters passed to the import function when reading
+#'   in the data from the files.
+#' @export
+#' @examples
+#'   # Use sample mhealth file for testing
+#'   filepaths = c(
+#'     system.file('extdata', 'mhealth.csv', package='MIMSunit')
+#'   )
+#'
+#'   # Test with multiple files
+#'   output = mims_unit_from_files(filepaths, epoch = "1 sec", dynamic_range = c(-8, 8))
+#'   head(output)
+mims_unit_from_files <-
+  function(files,
+           epoch = "5 sec",
+           dynamic_range,
+           output_mims_per_axis = FALSE,
+           use_gui_progress = FALSE,
+           file_type = "mhealth", ...) {
+    num_of_files <- length(files)
+    dots = list(...)
+    df <- NULL
+    results <- list()
+
+    if (file_type == "mhealth") {
+      import_fun <- import_mhealth_csv_chunked
+      header = TRUE
+    } else if (file_type == "actigraph") {
+      import_fun <- function(x, chunk_samples) import_actigraph_csv_chunked(x, chunk_samples = chunk_samples, ...)
+      header = dots$header
+    } else {
+      stop('Only "mhealth" or "actigraph" file types are supported')
+    }
+
+    meta = .get_meta(files[1], file_type = file_type, header = header)
+    sr = meta$sr
+    st = meta$st
+    num_samples_epoch = parse_epoch_string(epoch, sr)
+    num_per_load = max(num_samples_epoch, 180000)
+    last_epoch_st = NULL
+    last_chunk = NULL
+    j = 1
+    if (shiny::isRunning()) {
+      file_pb = shiny::Progress$new()
+    }
+    for (i in 1:num_of_files) {
+      if (shiny::isRunning()) {
+        file_pb$set(value=i/num_of_files, message=paste("Compute MIMS-unit values for", files[i]))
+      }
+      funcs = import_fun(files[i], chunk_samples = num_per_load)
+      next_chunk = funcs[[1]]
+      close_con = funcs[[2]]
+
+      if (shiny::isRunning()) {
+        chunk_pb = shiny::Progress$new()
+        chunk_pb$set(value=0, message="Compute MIMS-unit values for chunks")
+      }
+      repeat {
+        chunk = next_chunk()
+        if (nrow(chunk) > 0) {
+          if (shiny::isRunning()) {
+            chunk_pb$inc(amount=0.01)
+          }
+          num_per_df = nrow(chunk)
+          if (!is.null(df)) {
+            if (!is.null(last_epoch_st) & !is.null(last_chunk)) {
+              df = clip_data(last_chunk, start_time = last_epoch_st,
+                             stop_time = last_chunk[nrow(last_chunk),1])
+              df = rbind(df, chunk[1:num_per_df,])
+            } else {
+              df = chunk[1:num_per_df,]
+            }
+          } else {
+            df = chunk[1:num_per_df,]
+          }
+          result = mims_unit(df,
+                             before_df = NULL,
+                             after_df = NULL,
+                             epoch = epoch,
+                             dynamic_range = dynamic_range,
+                             output_mims_per_axis = output_mims_per_axis,
+                             use_gui_progress = use_gui_progress,
+                             st = st
+          )
+          last_epoch_st = result[nrow(result),1]
+          last_chunk = chunk
+          # discard the last epoch
+          result = result[1:(nrow(result) - 1),]
+          results[[j]] = result
+          j = j + 1
+        }
+        else {
+          if (shiny::isRunning()) {
+            chunk_pb$set(value=1)
+            chunk_pb$close()
+          }
+          close_con()
+          break
+        }
+      }
+    }
+    if (shiny::isRunning()) {
+      file_pb$close()
+    }
+    result <- do.call(rbind, results)
+    return(result)
+  }
+
+
+.get_meta <- function(filepath, file_type, header) {
+  if (file_type == 'actigraph') {
+    meta = import_actigraph_meta(filepath, header = header)
+    st = meta$st
+    sr = meta$sr
+    return(list(st = st, sr = sr))
+  } else if (file_type == 'mhealth') {
+    # use the first 200 rows to compute sampling rate
+    funcs = import_mhealth_csv_chunked(filepath, chunk_samples = 200)
+    next_chunk = funcs[[1]]
+    close_con = funcs[[2]]
+    df = next_chunk()
+    close_con()
+    sr = sampling_rate(df)
+    st = df[1, 1]
+    return(list(st = st, sr = sr))
+  }
+}
